' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE "SDL.INC"
#INCLUDE "GLU.INC"
#INCLUDE "GDIPUTILS.INC"

$WindowCaption = "NeHe Lesson 20"

GLOBAL hDC AS LONG                ' Device context handle
GLOBAL TextureHandles() AS DWORD  ' Texture handles
GLOBAL masking AS LONG            ' Masking on/off
GLOBAL scene AS LONG              ' Which scene to draw
GLOBAL roll AS SINGLE             ' Rolling texture

' =======================================================================================
' All the setup goes here
' =======================================================================================
SUB SetupScene ()

   LOCAL hr AS LONG
   LOCAL i AS LONG
   DIM   TextureHandles(4) AS DWORD
   DIM   TextureWidth(4) AS LONG
   DIM   TextureHeight(4) AS LONG
   DIM   strTextureData(4) AS STRING

   masking = %TRUE

   ' Load textures from disk
   hr = GdiPlusLoadTexture("logo.bmp", TextureWidth(0), TextureHeight(0), strTextureData(0), %TRUE)
   hr = GdiPlusLoadTexture("mask1.bmp", TextureWidth(1), TextureHeight(1), strTextureData(1), %TRUE)
   hr = GdiPlusLoadTexture("image1.bmp", TextureWidth(2), TextureHeight(2), strTextureData(2), %TRUE)
   hr = GdiPlusLoadTexture("mask2.bmp", TextureWidth(3), TextureHeight(3), strTextureData(3), %TRUE)
   hr = GdiPlusLoadTexture("image2.bmp", TextureWidth(4), TextureHeight(4), strTextureData(4), %TRUE)

   ' Create two textures
   glGenTextures 5, TextureHandles(0)
   FOR i = 0 TO 4
      glBindTexture %GL_TEXTURE_2D, TextureHandles(i)
      glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_LINEAR
      glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_LINEAR
      glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth(i), TextureHeight(i), 0, _
                   %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData(i))
   NEXT

   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Enables clearing of the depth buffer
   glClearDepth 1.0!
   ' Enable depth testing
   glEnable %GL_DEPTH_TEST
   ' Enables smooth color shading
   glShadeModel %GL_SMOOTH
   ' Enable 2D texture mapping
   glEnable %GL_TEXTURE_2D

END SUB
' =======================================================================================

' =======================================================================================
' Draw the scene
' =======================================================================================
SUB DrawScene (BYVAL nWidth AS LONG, BYVAL nHeight AS LONG)

   LOCAL i AS LONG
   LOCAL sx, sy, sz AS SINGLE

   ' Clear the screen buffer
   glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

   ' Select and setup the projection matrix
   glMatrixMode %GL_PROJECTION
   glLoadIdentity

   ' Prevent divide by zero making height equal one
   IF nHeight = 0 THEN nHeight = 1

   ' Calculate the aspect ratio of the window
   gluPerspective 45.0!, nWidth/nHeight, 1.0!, 100.0!

   ' Select and setup the modelview matrix
   glMatrixMode %GL_MODELVIEW
   glLoadIdentity

   ' Move into the screen 5 units
   glTranslatef 0.0!, 0.0!, -2.0!

   ' Select the logo texture
   glBindTexture %GL_TEXTURE_2D, TextureHandles(0)
   ' Start drawing a textured quad
   glBegin %GL_QUADS
      glTexCoord2f 0.0!, - roll + 0.0! : glVertex3f -1.1!, -1.1!,  0.0!  ' Bottom left
      glTexCoord2f 3.0!, - roll + 0.0! : glVertex3f  1.1!, -1.1!,  0.0!  ' Bottom right
      glTexCoord2f 3.0!, - roll + 3.0! : glVertex3f  1.1!,  1.1!,  0.0!  ' Top right
      glTexCoord2f 0.0!, - roll + 3.0! : glVertex3f -1.1!,  1.1!,  0.0!  ' Top left
   glEnd

   glEnable %GL_BLEND                           ' Enable blending
   glDisable %GL_DEPTH_TEST                     ' Disable depth testing

   IF masking THEN                              ' Is masking enabled?
      glBlendFunc %GL_DST_COLOR, %GL_ZERO       ' Blend screen color with zero (black)
   END IF

   IF scene THEN                                ' Are we drawing the second scene?
      glTranslatef 0.0!, 0.0!, -1.0!            ' Translate into the screen one unit
      glRotatef roll * 360, 0.0!, 0.0!, 1.0!    ' Rotate on the z axis 360 degrees
      IF masking THEN                           ' Is masking on?
         glBindTexture %GL_TEXTURE_2D, TextureHandles(3)   ' Select the second mask texture
         glBegin %GL_QUADS                      ' Start drawing a textured quad
            glTexCoord2f 0.0!, 0.0! : glVertex3f -1.1!, -1.1!, 0.0!   ' Bottom left
            glTexCoord2f 1.0!, 0.0! : glVertex3f  1.1!, -1.1!, 0.0!   ' Bottom right
            glTexCoord2f 1.0!, 1.0! : glVertex3f  1.1!,  1.1!, 0.0!   ' Top right
            glTexCoord2f 0.0!, 1.0! : glVertex3f -1.1!,  1.1!, 0.0!   ' Top left
         glEnd                                  ' Done Drawing The Quad
      END IF

      glBlendFunc %GL_ONE, %GL_ONE                      ' Copy image 2 color to the screen
      glBindTexture %GL_TEXTURE_2D, TextureHandles(4)   ' Select the second image texture
      glBegin %GL_QUADS                                 ' Start drawing a textured quad
         glTexCoord2f 0.0!, 0.0! : glVertex3f -1.1!, -1.1!,  0.0!     ' Bottom left
         glTexCoord2f 1.0!, 0.0! : glVertex3f  1.1!, -1.1!,  0.0!     ' Bottom right
         glTexCoord2f 1.0!, 1.0! : glVertex3f  1.1!,  1.1!,  0.0!     ' Top right
         glTexCoord2f 0.0!, 1.0! : glVertex3f -1.1!,  1.1!,  0.0!     ' Top left
      glEnd                                             ' Done Drawing The Quad
   ELSE
      IF masking THEN                                     ' Is masking on?
         glBindTexture %GL_TEXTURE_2D, TextureHandles(1)  ' Select the first mask texture
         glBegin %GL_QUADS                                ' Start drawing a textured quad
            glTexCoord2f roll + 0.0!, 0.0! : glVertex3f -1.1!, -1.1!, 0.0!  ' Bottom left
            glTexCoord2f roll + 4.0!, 0.0! : glVertex3f  1.1!, -1.1!, 0.0!  ' Bottom right
            glTexCoord2f roll + 4.0!, 4.0! : glVertex3f  1.1!,  1.1!, 0.0!  ' Top right
            glTexCoord2f roll + 0.0!, 4.0! : glVertex3f -1.1!,  1.1!, 0.0!  ' Top left
         glEnd                                            ' Done drawing the quad
      END IF

      glBlendFunc %GL_ONE, %GL_ONE                        ' Copy image 1 color to the screen
      glBindTexture %GL_TEXTURE_2D, TexTureHandles(2)     ' Select the first image texture
      glBegin %GL_QUADS                                   ' Start drawing a textured quad
         glTexCoord2f roll + 0.0!, 0.0! : glVertex3f -1.1!, -1.1!, 0.0!  ' Bottom left
         glTexCoord2f roll + 4.0!, 0.0! : glVertex3f  1.1!, -1.1!, 0.0!  ' Bottom right
         glTexCoord2f roll + 4.0!, 4.0! : glVertex3f  1.1!,  1.1!, 0.0!  ' Top right
         glTexCoord2f roll + 0.0!, 4.0! : glVertex3f -1.1!,  1.1!, 0.0!  ' Top left
      glEnd                                               ' Done drawing the quad
   END IF

   glEnable %GL_DEPTH_TEST                    ' Enable depth testing
   glDisable %GL_BLEND                        ' Disable blending

   roll = roll + 0.002!                       ' Increase our texture roll bariable
   IF roll > 1.0! THEN                        ' Is roll greater than one
      roll = roll - 1.0!                      ' Subtract 1 from roll
   END IF

   ' // Swap buffers
   SDL_GL_SwapBuffers

END SUB
' =======================================================================================

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL pscreen AS SDL_Surface PTR
   LOCAL nWidth, nHeight, bpp AS LONG
   LOCAL lResult, bFullScreen AS LONG
   LOCAL szCaption AS ASCIIZ * 256
   LOCAL dwFlags AS DWORD
   LOCAL t, t0, fps AS DOUBLE
   LOCAL nFrames AS LONG
   LOCAL mp, sp AS LONG

   szCaption = $WindowCaption   ' Window caption

	' Ask the user which screen mode he prefers
	lResult = MessageBox(%NULL, "Would you like to run in fullscreen mode?", _
             "Start fullScreen?", %MB_YESNOCANCEL OR %MB_ICONQUESTION)
   SELECT CASE lResult
      CASE %IDCANCEL : EXIT FUNCTION
      CASE %IDYES    : bFullScreen = %TRUE
      CASE %IDNO     : bFullScreen = %FALSE
   END SELECT

   ' // Initialize SDL's subsystems - in this case, only video.
   IF SDL_Init(%SDL_INIT_VIDEO) < 0 THEN
      MSGBOX "Unable to init SDL: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Attempt to create a 640x480 window
   nWidth = 640 : nHeight = 480 : bpp = 16
   IF bFullScreen THEN
      dwFlags = %SDL_OPENGL OR %SDL_FULLSCREEN
   ELSE
      dwFlags = %SDL_OPENGL
   END IF
   pscreen = SDL_SetVideoMode(nWidth, nHeight, bpp, dwFlags)
   SDL_WM_SetCaption szCaption, ""

   ' // If we fail, return error.
   IF pscreen = %NULL THEN
      SDL_Quit
      MSGBOX "Unable to set the video mode: " & SDL_GetError()
      EXIT FUNCTION
   END IF

   ' // Set up the scene
   SetupScene

   ' Enable keyboard repeat
   SDL_EnableKeyRepeat %SDL_DEFAULT_REPEAT_DELAY, %SDL_DEFAULT_REPEAT_INTERVAL

   ' // Main loop
   LOCAL done AS LONG
   LOCAL uevent AS SDL_Event
   WHILE done = %FALSE

      ' // Poll for events, and handle the ones we care about.
      WHILE SDL_PollEvent(VARPTR(uevent))
         SELECT CASE uevent.type
            CASE %SDL_KEYDOWN
               SELECT CASE uevent.key.keysym.sym
                  CASE %SDLK_SPACE
                     IF ISFALSE sp THEN
                        sp = %TRUE
                        scene = NOT scene
                     END IF
                  CASE %SDLK_M
                     IF ISFALSE mp THEN
                        mp = %TRUE
                        IF ISTRUE masking THEN masking = %FALSE ELSE masking = %TRUE
                     END IF
               END SELECT
            CASE %SDL_KEYUP
               ' // Quit if escape is pressed
               IF uevent.key.keysym.sym = %SDLK_ESCAPE THEN
                  done = %TRUE
                  EXIT LOOP
               END IF
               ' Reset flags
               mp = %FALSE
               sp = %FALSE
            CASE %SDL_QUIT
               done = %TRUE
               EXIT LOOP
         END SELECT
      WEND

      IF ISFALSE bFullScreen THEN
         ' Get time and mouse position
         t = INT(TIMER)
         ' Calculate and display FPS (frames per second)
         IF t > t0 OR nFrames = 0 THEN
            fps = nFrames \ (t - t0)
            wsprintf szCaption, $WindowCaption & " (%i FPS)", BYVAL fps
            SDL_WM_SetCaption szCaption, szCaption
            t0 = t
            nFrames = 0
         END IF
         nFrames = nFrames + 1
      END IF

      ' // Draw the scene
      DrawScene nWidth, nHeight

   WEND

   ' Delete the textures
   glDeleteTextures(5, TextureHandles(0))

   ' Shut down SDL
   SDL_Quit

END FUNCTION
' =======================================================================================
